; A memory game
; Designed to be assembled with Tasm 3.1
; Free for any use.
; - Benjamin Green <benjamin@lh.co.nz>

	.org	8000h
	ld	sp,0FFFFh
	ld	hl,30h	; DsEventClear
	ld	(0C000h),hl
	rst	10h
	di
	in	a,(3)
	ld	l,a
	in	a,(4)
	ld	h,a
	push	hl
	ld	a,i
	push	af
	ld	a,081h
	ld	i,a
	in	a,(7)
	push	af
	ld	a,0FFh-32	; allow 64Hz int only
	out	(7),a
	im	2
	ei
	call	main
	di
	im	1
	pop	af
	out	(7),a
	pop	af
	ld	i,a
	pop	hl
	ld	a,l
	out	(3),a
	ld	a,h
	out	(4),a
	ei
	ld	hl,42h	; DsAddinTerminate
	ld	(0C000h),hl
	rst	10h

	; handy free space

	.fill	08080h-$,0
; ------------------------------------------------------------
	di
	push	af
	ld	a,0FFh
	out	(6),a
	pop	af
	ei
	reti
; ------------------------------------------------------------

	; handy free space

	.fill	08100h-$,0
	.fill	257,080h



main:	ld	c,15h	; init sound
	in	d,(c)
	inc	c
	in	e,(c)
	push	de
	inc	c
	in	d,(c)
	inc	c
	in	e,(c)
	push	de
	in	a,(19h)
	push	af
	xor	a
	out	(16h),a
	out	(18h),a
	inc	a
	out	(19h),a

	ld	a,10h	; draw up screen
	out	(4),a
	xor	a
	out	(3),a
	ld	hl,0A000h
	ld	de,0A001h
	ld	bc,3599
	ld	(hl),0
	ldir
	call	quadrant
	inc	a
	call	quadrant
	inc	a
	call	quadrant
	inc	a
	call	quadrant

	ld	b,20
preprepause:	halt
	djnz	preprepause
	ld	hl,splash
	ld	de,0A52Bh
	ld	b,39
_splashy:	push	bc
	ld	bc,24
	ldir
	ex	de,hl
	ld	c,6
	add	hl,bc
	ex	de,hl
	pop	bc
	djnz	_splashy
	ld	b,80
splashpause:	halt
	djnz	splashpause
	ld	hl,0A52Bh
	ld	d,0
	ld	b,39
	xor	a
_clearsides:	ld	(hl),a
	inc	hl
	ld	(hl),a
	ld	e,21
	add	hl,de
	ld	(hl),a
	inc	hl
	ld	(hl),a
	ld	e,7
	add	hl,de
	djnz	_clearsides
	call	quadrant
	inc	a
	call	quadrant
	inc	a
	call	quadrant
	inc	a
	call	quadrant
	ld	b,30
prepause:	halt
	djnz	prepause



	ld	hl,0F000h	; init empty array
	push	hl
	ld	de,0F001h
	ld	bc,2047
	ld	(hl),0FFh
	ldir
	pop	de

	ld	b,40	; initial rate

gameloop:
	ld	a,r	; random quad
	and	3
	ld	(de),a
	inc	de

	ld	hl,0F000h
playseq:	ld	a,(hl)
	cp	0FFh
	jr	z,gotestuser
	call	flashquad
	inc	hl
	jp	playseq


gotestuser:	ld	hl,0F000h
testuser:	ld	a,(hl)
	cp	0FFh
	jp	z,nextround
	push	hl

	push	bc
	ld	b,128	; 2 sec timeout
_waittap:	in	a,(10h)
	and	a
	jp	nz,_getpos
	halt
	djnz	_waittap
	pop	bc
	pop	hl
	jp	endmain


_getpos:	pop	bc

	ld	a,2
	out	(61h),a
	ld	a,20h
	out	(64h),a
	ld	a,8
	out	(65h),a
	in	a,(68h)
	or	1
	out	(68h),a
_adcwait:	in	a,(68h)
	bit	4,a
	jp	z,_adcwait

	push	de

	ld	a,0

	ld	c,69h
	in	l,(c)
	inc	c
	in	h,(c)

	ld	de,540
	cp	a
	sbc	hl,de
	jp	nc,_horz
	inc	a
	inc	a

_horz:	inc	c
	in	l,(c)
	inc	c
	in	h,(c)

	ld	de,500
	cp	a
	sbc	hl,de
	jp	nc,_endgetpos
	inc	a

_endgetpos:	pop	de

	push	af
	add	a,4
	call	quadrant
	push	bc
	ld	b,5	; min tap beep length
_tapbeeppause:	halt
	djnz	_tapbeeppause
	pop	bc
_waitrelease:	halt
	in	a,(10h)
	and	a
	jp	nz,_waitrelease
	pop	af
	call	quadrant

	pop	hl
	cp	(hl)
	jp	nz,endmain
	inc	hl
	jp	testuser



nextround:
	push	bc
	ld	b,70
seqpause:	halt
	djnz	seqpause
	pop	bc

	ld	a,b
	cp	10
	jp	z,topspeed
	dec	b
topspeed:

	jp	gameloop



endmain:	ld	a,165
	out	(17h),a
	ld	a,2
	out	(16h),a
	ld	a,1
	ld	b,10
_failloop:	push	bc
	out	(16h),a
	ld	b,5
_failpause:	halt
	djnz	_failpause
	xor	1
	pop	bc
	djnz	_failloop

	pop	af
	out	(19h),a
	ld	c,18h
	pop	de
	out	(c),e
	dec	c
	out	(c),d
	dec	c
	ld	a,2
	out	(c),a
	pop	de
	out	(c),e
	dec	c
	out	(c),d
	ret


quadrant:	push	bc	; A=0..7 (A<4 turns off, A>=4 turns on)
	push	de
	push	hl

	ld	d,0	; hl=bmp+(a*600)
	ld	e,a
	sla	e
	sla	e
	sla	e
	ld	h,d
	ld	l,e
	add	hl,de
	add	hl,de
	add	hl,de
	add	hl,de
	ld	d,h
	ld	e,l
	add	hl,de
	add	hl,de
	add	hl,de
	add	hl,de
	ld	d,h
	ld	e,l
	add	hl,de
	add	hl,de
	ld	de,bmp
	add	hl,de

	push	hl	; de=0A005h+(10x)+(1800y)
	ld	hl,0A005h
	bit	0,a
	jr	z,_left
	ld	de,10
	add	hl,de
_left:	bit	1,a
	jr	z,_top
	ld	de,1800
	add	hl,de
_top:	ex	de,hl
	pop	hl

	ld	b,60
_nexty:	push	bc
	ld	bc,10
	ldir
	ex	de,hl
	ld	c,20
	add	hl,bc
	ex	de,hl
	pop	bc
	djnz	_nexty

	push	af
	bit	2,a
	jp	z,_soundoff
	ld	hl,notes-4
	ld	d,0
	ld	e,a
	add	hl,de
	ld	a,(hl)
	out	(17h),a
	ld	a,2
	out	(16h),a
	ld	a,1
	out	(16h),a
	jp	_endsound
_soundoff:	xor	a
	out	(16h),a
_endsound:	pop	af

	pop	hl
	pop	de
	pop	bc
	ret


flashquad:	push	bc	; A=quadrant (0..3), B=delay
	add	a,4
	call	quadrant
	push	bc
_onpause:	halt
	djnz	_onpause
	pop	bc
	sub	4
	call	quadrant
	srl	b
	srl	b
_offpause:	halt
	djnz	_offpause
	pop	bc
	ret


notes	.db	64,54,82,40

#include "graphics.asm"

	.end
